@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.c 5662 2007-06-28 23:26:53Z sfre $
+ * $Id: gedit-document.c 5803 2007-08-15 00:08:37Z paolo $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -98,6 +98,7 @@ struct _GeditDocumentPrivate
 	gint	     last_save_was_manually : 1; 	
 	gint	     language_set_by_user : 1;
 	gint         stop_cursor_moved_emission : 1;
+	gint         dispose_has_run : 1;
 
 	gchar	    *uri;
 	gint 	     untitled_number;
@@ -205,19 +206,17 @@ release_untitled_number (gint n)
 }
 
 static void
-gedit_document_finalize (GObject *object)
+gedit_document_dispose (GObject *object)
 {
 	GeditDocument *doc = GEDIT_DOCUMENT (object); 
-	
+
 	gedit_debug (DEBUG_DOCUMENT);
 
-	if (doc->priv->untitled_number > 0)
-	{
-		g_return_if_fail (doc->priv->uri == NULL);
-		release_untitled_number (doc->priv->untitled_number);
-	}
-
-	if (doc->priv->uri != NULL)
+	/* Metadata must be saved here and not in finalize
+	 * because the language is gone by the time finalize runs.
+	 * beside if some plugin prevents proper finalization by
+	 * holding a ref to the doc, we still save the metadata */
+	if ((!doc->priv->dispose_has_run) && (doc->priv->uri != NULL))
 	{
 		GtkTextIter iter;
 		gchar *position;
@@ -247,17 +246,37 @@ gedit_document_finalize (GObject *object)
 		}
 	}
 
+	if (doc->priv->loader)
+	{
+		g_object_unref (doc->priv->loader);
+		doc->priv->loader = NULL;
+	}
+
+	doc->priv->dispose_has_run = TRUE;
+
+	G_OBJECT_CLASS (gedit_document_parent_class)->dispose (object);
+}
+
+static void
+gedit_document_finalize (GObject *object)
+{
+	GeditDocument *doc = GEDIT_DOCUMENT (object); 
+
+	gedit_debug (DEBUG_DOCUMENT);
+
+	if (doc->priv->untitled_number > 0)
+	{
+		g_return_if_fail (doc->priv->uri == NULL);
+		release_untitled_number (doc->priv->untitled_number);
+	}
+
 	g_free (doc->priv->uri);
 	if (doc->priv->vfs_uri != NULL)
 		gnome_vfs_uri_unref (doc->priv->vfs_uri);
 
 	g_free (doc->priv->mime_type);
-
-	if (doc->priv->loader)
-		g_object_unref (doc->priv->loader);
-
 	g_free (doc->priv->search_text);
-	
+
 	if (doc->priv->to_search_region != NULL)
 	{
 		/* we can't delete marks if we're finalizing the buffer */
@@ -366,6 +385,7 @@ gedit_document_class_init (GeditDocumentClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkTextBufferClass *buf_class = GTK_TEXT_BUFFER_CLASS (klass);
 
+	object_class->dispose = gedit_document_dispose;
 	object_class->finalize = gedit_document_finalize;
 	object_class->get_property = gedit_document_get_property;
 	object_class->set_property = gedit_document_set_property;	
@@ -583,6 +603,32 @@ set_encoding (GeditDocument       *doc,
 	g_object_notify (G_OBJECT (doc), "encoding");
 }
 
+static GtkSourceStyleScheme *
+get_default_style_scheme (void)
+{
+	gchar                 *scheme_id;
+	GtkSourceStyleScheme  *def_style;
+	GtkSourceStyleManager *manager;
+	
+	manager = gedit_get_source_style_manager ();
+	scheme_id = gedit_prefs_manager_get_source_style_scheme ();
+	def_style = gtk_source_style_manager_get_scheme (manager, scheme_id);
+
+	if (def_style == NULL)
+	{
+		g_warning ("Default style scheme '%s' cannot be found, falling back to 'classic' style scheme ", scheme_id);
+			
+		def_style = gtk_source_style_manager_get_scheme (manager, "classic");
+		if (def_style == NULL) 
+		{
+			g_warning ("Style scheme 'classic' cannot be found, check your GtkSourceView installation.");			
+		}
+	}
+
+	g_free (scheme_id);
+
+	return def_style;
+}
 static void
 gedit_document_init (GeditDocument *doc)
 {
@@ -606,6 +652,8 @@ gedit_document_init (GeditDocument *doc)
 	doc->priv->last_save_was_manually = TRUE;
 	doc->priv->language_set_by_user = FALSE;
 
+	doc->priv->dispose_has_run = FALSE;
+
 	doc->priv->mtime = 0;
 
 	g_get_current_time (&doc->priv->time_of_last_save_or_load);
@@ -623,9 +671,10 @@ gedit_document_init (GeditDocument *doc)
 						       gedit_prefs_manager_get_enable_search_highlighting ());
 
 	style_manager = gedit_get_source_style_manager ();
-	style_scheme = gedit_source_style_manager_get_default_scheme (style_manager);
-	gtk_source_buffer_set_style_scheme (GTK_SOURCE_BUFFER (doc),
-					    style_scheme);
+	style_scheme = get_default_style_scheme ();
+	if (style_scheme != NULL)
+		gtk_source_buffer_set_style_scheme (GTK_SOURCE_BUFFER (doc),
+						    style_scheme);
 
 	g_signal_connect_after (doc, 
 			  	"insert-text",
@@ -1090,7 +1139,7 @@ gedit_document_load (GeditDocument       *doc,
 	g_return_if_fail (doc->priv->loader == NULL);
 
 	/* create a loader. It will be destroyed when loading is completed */
-	doc->priv->loader = gedit_document_loader_new (doc);
+	doc->priv->loader = gedit_document_loader_new (doc, uri, encoding);
 
 	g_signal_connect (doc->priv->loader,
 			  "loading",
@@ -1103,9 +1152,7 @@ gedit_document_load (GeditDocument       *doc,
 
 	set_uri (doc, uri, NULL);
 
-	gedit_document_loader_load (doc->priv->loader,
-				    uri,
-				    encoding);
+	gedit_document_loader_load (doc->priv->loader);
 }
 
 gboolean
@@ -1190,7 +1237,7 @@ gedit_document_save_real (GeditDocument          *doc,
 	g_return_if_fail (doc->priv->saver == NULL);
 
 	/* create a saver, it will be destroyed once saving is complete */
-	doc->priv->saver = gedit_document_saver_new (doc);
+	doc->priv->saver = gedit_document_saver_new (doc, uri, encoding, flags);
 
 	g_signal_connect (doc->priv->saver,
 			  "saving",
@@ -1200,10 +1247,7 @@ gedit_document_save_real (GeditDocument          *doc,
 	doc->priv->requested_encoding = encoding;
 
 	gedit_document_saver_save (doc->priv->saver,
-				   uri,
-				   encoding,
-				   doc->priv->mtime,
-				   flags);
+				   doc->priv->mtime);
 }
 
 /**
@@ -1759,6 +1803,110 @@ _gedit_document_get_seconds_since_last_save_or_load (GeditDocument *doc)
 	return (current_time.tv_sec - doc->priv->time_of_last_save_or_load.tv_sec);
 }
 
+static void
+get_search_match_colors (GeditDocument *doc,
+			 gboolean      *foreground_set,
+			 GdkColor      *foreground,
+			 gboolean      *background_set,
+			 GdkColor      *background)
+{
+	GtkSourceStyleScheme *style_scheme;
+	GtkSourceStyle *style;
+	gchar *bg;
+	gchar *fg;
+
+	style_scheme = gtk_source_buffer_get_style_scheme (GTK_SOURCE_BUFFER (doc));
+	if (style_scheme == NULL)
+		goto fallback;
+
+	style = gtk_source_style_scheme_get_style (style_scheme,
+						   "search-match");
+	if (style == NULL)
+		goto fallback;
+
+	g_object_get (style, 
+		      "foreground-set", foreground_set, 
+		      "foreground", &fg,
+		      "background-set", background_set, 
+		      "background", &bg,
+		      NULL);
+
+	if (*foreground_set)
+	{
+		if (fg == NULL ||
+		    !gdk_color_parse (fg, foreground))
+		{
+			*foreground_set = FALSE;
+		}
+	}
+
+	if (*background_set)
+	{
+		if (bg == NULL ||
+		    !gdk_color_parse (bg, background))
+		{
+			*background_set = FALSE;
+		}
+	}	
+
+	g_free (fg);
+	g_free (bg);
+
+	return;
+
+ fallback:
+	gedit_debug_message (DEBUG_DOCUMENT, 
+			     "Falling back to hard-coded colors "
+			     "for the \"found\" text tag.");
+
+	gdk_color_parse ("#FFFF78", background);
+	*background_set = TRUE;
+	*foreground_set = FALSE;
+
+	return;
+}
+
+static void
+sync_found_tag (GeditDocument *doc,
+		GParamSpec    *pspec,
+		gpointer       data)
+{
+	GdkColor fg;
+	GdkColor bg;
+	gboolean fg_set;
+	gboolean bg_set;
+
+	gedit_debug (DEBUG_DOCUMENT);
+
+	g_return_if_fail (GTK_TEXT_TAG (doc->priv->found_tag));
+
+	get_search_match_colors (doc,
+				 &fg_set, &fg,
+				 &bg_set, &bg);
+
+	if (fg_set)
+		g_object_set (doc->priv->found_tag,
+			      "foreground-gdk", &fg,
+			      NULL);
+
+	if (bg_set)
+		g_object_set (doc->priv->found_tag,
+			      "background-gdk", &bg,
+			      NULL);
+}
+
+static void
+text_tag_set_highest_priority (GtkTextTag    *tag,
+			       GtkTextBuffer *buffer)
+{
+	GtkTextTagTable *table;
+	gint n;
+
+	table = gtk_text_buffer_get_tag_table (buffer);
+	n = gtk_text_tag_table_get_size (table);
+	gtk_text_tag_set_priority (tag, n - 1);
+}
+
 static void
 search_region (GeditDocument *doc,
 	       GtkTextIter   *start,
@@ -1778,14 +1926,24 @@ search_region (GeditDocument *doc,
 	
 	if (doc->priv->found_tag == NULL)
 	{
-		/* FIXME: the colors are hardcoded */
 		doc->priv->found_tag = gtk_text_buffer_create_tag (GTK_TEXT_BUFFER (doc), 
 								   "found",
-								   "background", "#FFFF78",
-								   //"foreground", "black",
 								   NULL);
+
+		sync_found_tag (doc, NULL, NULL);
+
+		g_signal_connect (doc, 
+				  "notify::style-scheme",
+				  G_CALLBACK (sync_found_tag),
+				  NULL);
 	}
 
+	/* make sure the 'found' tag has the priority over
+	 * syntax highlighting tags */
+	text_tag_set_highest_priority (doc->priv->found_tag,
+				       GTK_TEXT_BUFFER (doc));
+				   
+
 	if (doc->priv->search_text == NULL)
 		return;
 		
