@@ -186,6 +186,7 @@ add_uri (GeditFileBookmarksStore * model, GnomeVFSURI * uri,
 		if (local)
 			name = tmp;
 		else {
+			/* Translators: this is used in "file on host", e.g. "/foo/bar on ftp.baz.org" */
 			name = g_strconcat(tmp, " ", _("on"), " ", gnome_vfs_uri_get_host_name (uri), NULL);
 			g_free (tmp);
 		}
@@ -328,7 +329,7 @@ process_volume (GeditFileBookmarksStore * model, GnomeVFSVolume * volume,
 		if (strcmp (uri, "file:///") == 0) {
 			*root = TRUE;
 
-			add_volume (model, volume, "File System",
+			add_volume (model, volume, _("File System"),
 				    GEDIT_FILE_BOOKMARKS_STORE_IS_ROOT |
 				    GEDIT_FILE_BOOKMARKS_STORE_IS_DRIVE,
 				    NULL);
